<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\User\User;
use Joomla\CMS\Component\ComponentHelper;

use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\UserRegState;
use LartsSubscriptions\Message;
use LartsSubscriptions\UserInfo;


class UserRegistration
{
	private $data;
	private $reset = 0;
	private $block = 0;
	private $activation = '';
	private $userid = 0;
  private $params = null;
  private $error = null;

	public function __construct()
	{
		// Getting params from com_users.
		$this->params = ComponentHelper::getParams('com_users');

		// Setting default block for the user.
    // see Joomla-Options -> Users -> Allow User Registration
		$this->block = ($this->params->get('useractivation') == 0) ? 0 : 1;
  }

	// Register a user in the Joomla! user database.
	public function setNewUser($username, $password, $name, $email)
	{
		$user = [
			'username' => ! empty($username) ? $username : $email,
			'password' => ! empty($password) ? $password : strtolower(uniqid()),
			'name'     => ! empty($name) ? $name : $email,
			'email'    => $email,
		];

		return $this->setNewUserByArray($user);
	}

	// Register a user in the Joomla! user database.
	public function setNewUserByArray($data = [], $bCheckAllowRegistration = true)
	{
    if($bCheckAllowRegistration) {
      if(!$this->params->get('allowUserRegistration')) {
        Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_SAVE_FAILED'), 'error');
        return false;
      }
    }

		if (empty($data['email']) || empty($data['name']) || empty($data['username']) || empty($data['password'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USER_SAVE_FAILED'), 'error');
			return false;
		}

    if ($this->isExsistingUser('email', $data['email'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_EMAIL_IN_DB'), 'error');
      return $this->userid;
		}
    if ($this->isExsistingUser('name', $data['username'])) {
			Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_USERNAME_IN_DB'), 'error');
			return $this->userid;
		}

		return $this->saveUserToJoomlaDatabase($data);
	}

	// Set user blocked by default.
	public function setBlockUser($block = 0)
	{
		$this->block = $block;
	}

	// Setting user reset on first login to yes.
	public function setResetRequiredOnFirstLogin($reset = 0)
	{
		$this->reset = $reset;
	}

	// Checks if the user is already present the database, if yes set the user id.
	public function isExsistingUser($identifier, $value)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['id'])
		            ->from($db->quoteName('#__users'))
		            ->where($db->quoteName($identifier) . ' = ' . $db->quote($value));
		$db->setQuery($query);
		$user = $db->loadResult();

		if(!empty($user)) {
			$this->userid = $user->id;
			return true;
		}

		return false;
	}

	// Saving the use to the Joomla user database and return its ID.
	private function saveUserToJoomlaDatabase($userdata = [])
	{
    $user = new User;
		$data = (array) $this->getUserData();

		foreach ($userdata as $k => $v)
		{
			$data[$k] = $v;
		}

		$data['block']        = $this->block;
		$data['activation']   = $this->activation;
		$data['requireReset'] = $this->reset;

		if(!$user->bind($data)) {
			$this->error = $user->getError();
			return false;
		}


    $bRet = $user->save();
    if(!$bRet) {
			$this->error = $user->getError();
			return false;
		}

		$this->userid = ! empty($user->id) ? $user->id : 0;

		return $user->id;
	}

	// Getting the userdata from the userstate if required and return it with its values or the defaults.
	private function getUserData()
	{
		$this->data = new \stdClass;

		$app    = Factory::getApplication();
		$params = ComponentHelper::getParams('com_users');


		$temp = UserRegState::getStateData();

		foreach ($temp as $k => $v)
		{
			$this->data->$k = $v;
		}

		// Get the groups the user should be added to after registration.
		$this->data->groups = isset($this->data->groups) ? array_unique($this->data->groups) : [];

		// Get the default new user group, Registered if not specified
		$this->data->groups[] = $params->get('new_usertype', 2);

		// Unset the passwords.
		unset($this->data->password1);
		unset($this->data->password2);

		// Get the results from the onContentPrepare functionality in the user plugins.
		$results = PluginManager::dispatchPrepareDataEvent('', 'user', ['com_lartssubscriptions.registration', $this->data], true);

		// Check for errors encountered while preparing the data.
		if (count($results) && in_array(false, $results, true)) {
			$this->data = false;
		}

		return $this->data;
	}

	// Actiavte and unblock the user by a given user id.
	public function ActivateUserByUserId($userid = 0)
	{
		if(!$userid) {
			return false;
		}

		$data = [
			'block'      => 0,
		];

		return $this->updateJoomlaUserInformation($userid, $data);
	}

	// Updating the user in the Joomla user manager.
	public function updateJoomlaUserInformation($userid = 0, $data = [])
	{
		// instantiate the user class.
		$user = new User($userid);

		if(!$user->bind($data)) {
			$this->error = $user->getError();
			return false;
		}

		if(!$user->save()) {
			$this->error = $user->getError();
			return false;
		}

		return true;
	}

	// Returns the erorr during the registration.
	public function getError()
	{
		return ! empty($this->error) ? $this->error : null;
	}

	// Returns the user id which has been created a few seconds ago.
	public function getuserId()
	{
		return $this->userid;
	}
	
	public static function removeUser($userid)
	{
		PluginManager::execPlugins('lartssubscriptions', 'onBeforeDeleteUser', $userid);

		// remove user details from our table
    UserInfo::remove($userid);
		
		// delete joomla user
		$user = new User($userid);
		if($user->id != 0) {
			$user->delete();
		}
	}

  // Sending a registration email to the newly created customer.
	public static function sendUserRegistration($ordercode, $userid)
	{
		// Variables for the message:
		$variables = [
			'password'  => '*********',
			'ordercode' => $ordercode,
			'date'      => 'NOW',
		];

		Message::getInstance('account-created')
		             ->user($userid)
		             ->variables($variables)
		             ->send();

		return true;
	}
}

