<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Http\HttpFactory;

ini_set("default_socket_timeout", 2);

use Joomla\CMS\Factory;

class Vat
{
	private $options    = [];
	private $vatNumber  = null;
	private $country    = null;
	private $client     = null;
	private $validation = false;
	private $message    = null;
	private $response   = [];
  private $requestType = null;

	private $api = 'http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl';
	private $url = 'https://controleerbtwnummer.eu/api/validate/';

	public function __construct($options = [])
	{
		foreach ($options as $option => $value)
		{
			$this->options[$option] = $value;
		}

		$this->client = $this->getSoapClient();

		$this->requestType = ! empty($this->client) ? 'soap' : 'api';
	}

	public function setRequestType($request = 'soap')
	{
		$this->requestType = $request;
		return $this;
	}

	public function setCountryCode($country)
	{
		$this->country = $country;

		return $this;
	}

	public function setVatNumber($vatNumber)
	{
		$this->vatNumber = $vatNumber;

		return $this;
	}

	public function validateVatNumber()
	{
		if (empty($this->vatNumber) || empty($this->country))
		{
			return $this;
		}

		$this->checkEuVatNumber($this->country, $this->vatNumber);

		return $this;
	}

	public function checkEuVatNumber($countryCode, $vatNumber)
	{
		$session = Factory::getSession()->get('VatValidation' . $countryCode . $vatNumber);
		$results = json_decode($session);

		if (!empty($results) && $results->countryCode == $countryCode && $results->vatNumber == $vatNumber)
		{
			$this->response     = $results;
			$this->validation = true;

			return $this;
		}

		// Checks if the number if matching a pattern.
		if(!$this->validateVatFormat($countryCode, $vatNumber))
		{
			$this->message = Text::_('COM_LARTSSUBS_VATNUMBER_FORMAT_IS_INCORRECT');

			return $this;
		}

		// Perform API request.
		if ($this->requestType == 'api')
		{
			$this->checkVatNumberAtWebApi($countryCode, $vatNumber);

			return $this;
		}

		// Perform WDSL check at the official VIES services.
		$this->checkVatNumberAtViesService($countryCode, $vatNumber);

		return $this;
	}

	/**
	 * Checking the VAT Number at https://controleerbtwnummer.eu
	 * This is a free service for VAT validation.
	 */
	private function checkVatNumberAtWebApi($countryCode, $vatNumber)
	{
		try
		{
			$results = HttpFactory::getHttp()->get($this->url . '/' . $countryCode . $vatNumber . '.json', null, 15)->body;

			if (empty($results))
			{
				$this->message = Text::_('COM_LARTSSUBS_VAT_RESPONSE_SERVICE_UNAVAILABLE');

				return $this;
			}

			$results = json_decode($results);
		}
		catch (\Exception $e)
		{
			$this->message = Text::_('COM_LARTSSUBS_VAT_RESPONSE_SERVICE_UNAVAILABLE');

			return $this;
		}

		$this->validation = $results->valid;

		$this->response = [
			'valid'       => $results->valid,
			'countryCode' => $countryCode,
			'vatNumber'   => $vatNumber,
			'name'        => $this->cleanUpString($results->name),
			'address'     => $this->cleanUpString($results->strAddress),
			'requestType' => $this->requestType,
		];

		Factory::getSession()->set('VatValidation' . $countryCode . $vatNumber, json_encode($this->response));

		return $this;
	}

	/**
	 * Checking the vat Registration at the official VAT Servers
	 *
	 * 'INVALID_INPUT'       => 'The provided CountryCode is invalid or the VAT number is empty',
	 * 'SERVICE_UNAVAILABLE' => 'The SOAP service is unavailable, try again later',
	 * 'MS_UNAVAILABLE'      => 'The Member State service is unavailable, try again later or with another Member State',
	 * 'TIMEOUT'             => 'The Member State service could not be reached in time, try again later or with another Member State',
	 * 'SERVER_BUSY'         => 'The service cannot process your request. Try again later.'
	 *
	 */
	private function checkVatNumberAtViesService($countryCode, $vatNumber)
	{
		try
		{
			$rs = $this->client->checkVat(['countryCode' => $countryCode, 'vatNumber' => $vatNumber]);
		}
		catch (\SoapFault $e)
		{
			if ($e->faultstring == 'INVALID_INPUT')
			{
				$this->message = Text::_('COM_LARTSSUBS_VAT_RESPONSE_INVALID_INPUT');

				return $this;
			}

			// Try via direct url if soap fails for other reason than invalid input
			if ($this->checkVatNumberAtWebApi($countryCode, $vatNumber))
			{
				return $this;
			}

			switch ($e->faultstring)
			{
				case 'SERVICE_UNAVAILABLE':
				case 'MS_UNAVAILABLE':

					$this->message = Text::_('COM_LARTSSUBS_VAT_RESPONSE_SERVICE_UNAVAILABLE');

					return $this;

				case 'TIMEOUT':
				case 'SERVER_BUSY':

					$this->message = Text::_('COM_LARTSSUBS_VAT_RESPONSE_SERVICE_SLOW');

					return $this;

				default:
					return $this;
			}
		}

		if ($this->isDebug())
		{
			$this->trace('Web Service result', $this->client->__getLastResponse());
		}

		if(!$rs->valid)
		{
			return $this;
		}

		$this->validation = true;

		$this->response = [
			'valid'       => $this->validation,
			'countryCode' => $countryCode,
			'vatNumber'   => $vatNumber,
			'name'        => $this->cleanUpString($rs->name),
			'address'     => $this->cleanUpString($rs->address),
			'requestType' => $this->requestType,
		];

		Factory::getSession()->set('VatValidation' . $countryCode . $vatNumber, json_encode($this->response));

		return $this;
	}

	private function trace($title, $body)
	{
		echo '<div class="alert alert-error">'
			. 'EU VIES server is down. Cannot check VAT numbers.<br>'
			. 'Please try again later'
			. '</div>';
	}
  
	/**
	 * Checking if the SOAP server has been installed.
	 */
	private function getSoapClient()
	{
		if(!is_null($this->client)) {
			return $this->client;
		}

		if(!class_exists('SoapClient')) {
			$this->client = 'api';
			return false;
		}

		$client = null;

		try
		{
			$context = stream_context_create(['http' => ['user_agent' => 'PHPSoapClient']]);

			$client = new \SoapClient(
				$this->api,
				[
					'trace'              => true,
					'stream_context'     => $context,
					'cache_wsdl'         => WSDL_CACHE_NONE,
					'connection_timeout' => 2,
				]
			);
		}
		catch (\Exception $e) {
			$this->serverDown('Vat Translation Error', $e->getMessage());
		}

		$this->client = $client ? : false;

		return $this->client;
	}

	private function cleanUpString($string)
	{
		// replace <br> with comma's
		$string = trim(str_replace('<br>', ', ', $string));

		// remove comma's at start/end of string
		$string = trim($string, ',');

		// remove duplicate spaces
		$string = preg_replace('#\s\s+#s', ' ', $string);

		// Convert string to TitleCase
		$string = ucwords(strtolower($string));

		return $string;
	}

	/**
	 * Checking if the format of the VIES registration is correct.
	 */
	public function validateVatFormat($country, $vatNumber)
	{
		$country_id = $this->getEUVatPatterns($country);

		if(!isset($country_id) && ! empty($country_id))
		{
			return false;
		}

		$matches = preg_match('/^' . $this->getEUVatPatterns($country) . '$/', $vatNumber) > 0;

		return $matches;
	}

	private function serverDown($title, $body)
	{
		echo '<div class="alert alert-error">'
			. 'EU VIES server is down. Cannot check VAT numbers.<br>'
			. 'Please try again later'
			. '</div>';
	}

	/**
	 * Sets the debug value from the options which are sent.
	 */
	private function isDebug()
	{
		return ($this->options['debug'] === true);
	}

	/**
	 * Giving back the results of the validation.
	 */
	public function isValid()
	{
		return $this->validation;
	}

	/**
	 * Giving back the name of the company.
	 */
	public function getName()
	{
		return isset($this->response['name']) ? $this->response['name'] : null;
	}

	/**
	 * Giving back the address of the customer.
	 */
	public function getAddress()
	{
		return isset($this->response['address']) ? $this->response['address'] : null;
	}

	/**
	 * Returning the response of a VAT number check.
	 */
	public function getResponse()
	{
		return $this->response;
	}

	/**
	 * Returning the message of a VAT number check.
	 */
	public function getMessage()
	{
		return ! empty($this->message) ? $this->message : null;
	}

	/**
	 * This holds the formats of VAT numbers in the EU which needs t be checked before sending them to an API.
	 */
	private function getEUVatPatterns($country)
	{
		$pattern = [
			'AT' => 'U[A-Z\d]{8}',
			'BE' => '(0\d{9}|\d{10})',
			'BG' => '\d{9,10}',
			'CY' => '\d{8}[A-Z]',
			'CZ' => '\d{8,10}',
			'DE' => '\d{9}',
			'DK' => '(\d{2} ?){3}\d{2}',
			'EE' => '\d{9}',
			'EL' => '\d{9}',
			'ES' => '[A-Z]\d{7}[A-Z]|\d{8}[A-Z]|[A-Z]\d{8}',
			'FI' => '\d{8}',
			'FR' => '([A-Z]{2}|\d{2})\d{9}',
			'GB' => '\d{9}|\d{12}|(GD|HA)\d{3}',
			'HR' => '\d{11}',
			'HU' => '\d{8}',
			'IE' => '[A-Z\d]{8}|[A-Z\d]{9}',
			'IT' => '\d{11}',
			'LT' => '(\d{9}|\d{12})',
			'LU' => '\d{8}',
			'LV' => '\d{11}',
			'MT' => '\d{8}',
			'NL' => '\d{9}B\d{2}',
			'PL' => '\d{10}',
			'PT' => '\d{9}',
			'RO' => '\d{2,10}',
			'SE' => '\d{12}',
			'SI' => '\d{8}',
			'SK' => '\d{10}',
		];

		if(!array_key_exists($country, $pattern))
		{
			return null;
		}

		return $pattern[$country];
	}
}