<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class BundledProducts
{
	/**
	 * Check if a product is grouped by one or more parents
	 * You can define how many levels deep it must search
	 *
	 * @param  int       $id      The id of the product
	 * @param  int|array $parents The ids of the parent products to search for
	 * @param  int       $depth   The number of levels to iterate through. Set to 0 to return all levels
	 *
	 * @return  array   An array with the product ids as keys and product names as values
	 */
	public static function hasParents($id, $parents, $depth = 0)
	{
		$parents = is_array($parents) ? $parents : [(int) $parents];

		$parents_found = self::getParentsById($id, $depth);

		return array_intersect($parents, $parents_found);
	}

	/**
	 * Check if a product has one or more parents grouped under it
	 * You can define how many levels deep it must search
	 *
	 * @param  int       $id       The id of the product
	 * @param  int|array $children The ids of the child products to search for
	 * @param  int       $depth    The number of levels to iterate through. Set to 0 to return all levels
	 *
	 * @return  array   An array with the product ids as keys and product names as values
	 */
	public static function hasChildren($id, $children, $depth = 0)
	{
		$children = is_array($children) ? $children : [(int) $children];

		$children_found = self::getChildrenById($id, $depth);

		return array_intersect($children, $children_found);
	}

	/**
	 * Get all parents of the given product id
	 * You can define how many levels deep it must search
	 *
	 * @param  int $id    The id of the product
	 * @param  int $depth The number of levels to iterate through. Set to 0 to return all levels
	 *
	 * @return  array   An array with the product ids as keys and product names as values
	 */
	public static function getParentsById($id, $depth = 0)
	{
		return self::getParentsOrChildrenById('parents', $id, $depth);
	}

	/**
	 * Get all children of the given product id
	 * You can define how many levels deep it must search
	 *
	 * @param  int $id    The id of the product
	 * @param  int $depth The number of levels to iterate through. Set to 0 to return all levels
	 *
	 * @return  array   An array with the product ids as keys and product names as values
	 */
	public static function getChildrenById($id, $depth = 0)
	{
		return self::getParentsOrChildrenById('children', $id, $depth);
	}

	/**
	 * Get all parents or children of the given product id
	 * You can define how many levels deep it must search
	 *
	 * @param  string $type  Either 'parents' or 'children'
	 * @param  int    $id    The id of the product
	 * @param  int    $depth The number of levels to iterate through. Set to 0 to return all levels
	 *
	 * @return  array   An array with the product ids as keys and product names as values
	 */
	private static function getParentsOrChildrenById($type, $id = 0, $depth = 0)
	{
		if(!$id) {
			return [];
		}

		$db = Factory::getDbo();

		$return     = [];
		$search_ids = [$id];

		$current_depth = 0;
		while ( ! empty($search_ids))
		{
			$select_id = $type == 'children' ? 'bundled_product_id' : 'product_id';
			$search_id = $type == 'children' ? 'product_id' : 'bundled_product_id';

			$query = $db->getQuery(true)
				->select(['a.'.$select_id. ' AS id', 'prod.name'])
				->from($db->quoteName('#__larts_subs_products_bundled', 'a'))
				->join('LEFT', $db->quoteName('#__larts_subs_products', 'prod') . ' ON (' . $db->quoteName('prod.id') . ' = ' . $db->quoteName('a.' . $select_id) . ')')
				->where($db->quoteName('a.' . $search_id) . ' IN (' . implode(',', $search_ids) . ')')
				->where($db->quoteName('prod.published') . ' = 1')
				->order($db->quoteName('prod.name') . ' ASC');

			$db->setQuery($query);
			$result = $db->loadAssocList('id', 'name');

			// filter out any ids already found
			$search_ids = array_diff(array_keys($result), array_keys($return));

			// add all found ids to the return list
			$return += $result;

			if ($depth && ++$current_depth >= $depth) {
				break;
			}
		}

		return $return;
	}
}
