<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Discount;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Customer;
use LartsSubscriptions\Base\Database;


class Coupon
{
  const TYPE_MANUAL = 0;
  const TYPE_AUTO = 1;
  
	var $config = null;
	var $ordercode = null;

  var $discount = null;

	var $coupon_code = null;
	var $coupon = null;
	var $coupons_disabled = 0;

	var $message = '';

	public function __construct($ordercode = '')
	{
		$this->config = LartsConfig::get();
		$this->ordercode = $ordercode ?: intval(Factory::getSession()->get('ordercode'));
		$this->discount = new Discount($this->ordercode);
  }

 	public function hasOrders()
	{
    $cart = new Cart;
		return $cart->getProductCount();
	}

	/**
	 * Check the coupon when it is coming from the URL.
	 * It will be passed by the main component file.
	 */
	public function storeCouponInSession($coupon_code = null)
	{
		if(!$coupon_code) {
			return;
		}

		$replace = [":", "/", "\\", "@", "#", "@", "!", "$", "?"];
		$coupon_code = str_replace($replace, "", $coupon_code);
		$coupon_code = strtoupper($coupon_code);

		// Setting the session as the coupon is valid and needs to be applied on order adds.
		Factory::getSession()->set('coupon_code', $coupon_code);

		// Shows a message that the coupon has been applied.
    $cart = new Cart();
		$message = $cart->isEmpty() ? 'COM_LARTSSUBS_COUPON_WILL_BEEN_APPLIED' : 'COM_LARTSSUBS_COUPON_HAS_BEEN_APPLIED';
		Factory::getApplication()->enqueueMessage(Text::sprintf($message, $coupon_code), 'success');

		return true;
	}

	public function apply(&$products, $coupon_code)
	{
		$this->coupon_code = $coupon_code ?: $this->coupon_code;
		$this->coupon = $this->getCoupon();

		if(!$this->check())
		{
      $cart = new Cart;
			$cart->removeCouponsFromOrder();
			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_COUPON_NO_LONGER_VALID', $this->coupon_code), 'notice');

			return;
		}

		if ($this->coupon->full_order || empty($this->coupon->product_ids)) {
			$this->applyFullOrderCoupon($products);

			return;
		}

		$this->applyProductBasedCoupon($products);
	}

	public function applyFullOrderCoupon(&$products)
	{
		$coupon_used          = false;
		$other_discounts_used = false;

		foreach ($products as &$product)
		{
			$coupon_discount = $this->discount->getDiscount(($product->price * $product->quantity), $this->coupon->discount, $this->coupon->discount_type);

			// Check if the renewal/upgrade discount should be used
			if (isset($product->discount_data) && $product->discount_data->discount > $coupon_discount)
			{
				$other_discounts_used = true;
				continue;
			}

			// Use the coupon code
			$coupon_used = true;
			$this->setCouponDiscountData($product, $coupon_discount);
		}

		// Coupon code discount is lower than the renewal/upgrade discounts
		if(!$coupon_used)
		{
			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_COUPON_LOWER_THAN_RENEWAL', $this->coupon_code), 'notice');

			return;
		}

		// Coupon code discount is higher than all the renewal/upgrade discounts
		// So remove all discounts and apply a full order discount
		if(!$other_discounts_used)
		{
			$this->resetDiscounts($products);
			$this->insertFullOrderDiscountRow($products);
		}
	}

	public function applyProductBasedCoupon(&$products)
	{
    // the entered coupon
		if (empty($products)) {
			return;
		}

		$coupon_productIDs = explode(',', $this->coupon->product_ids);

		foreach ($products as &$product)
    {
			if(!in_array($product->product_id, $coupon_productIDs)) {
				continue;
			}

			$coupon_discount = $this->discount->getDiscount($product->price, $this->coupon->discount, $this->coupon->discount_type);

			if(!isset($product->discount_data)) {
				$this->setCouponDiscountData($product, $coupon_discount);
				continue;
			}

			// Coupon code discount is lower than the renewal/upgrade discount
			if ($coupon_discount < $product->discount_data->discount) {
				Factory::getApplication()->enqueueMessage(Text::sprintf('COM_LARTSSUBS_COUPON_LOWER_THAN_RENEWAL_PRODUCT', $this->coupon_code, $product->name), 'notice');
				continue;
			}

			// Coupon code discount is higher than the renewal/upgrade discount
			$this->setCouponDiscountData($product, $coupon_discount);
		}
	}

	public function setCouponDiscountData(&$product, $discount)
	{
		$product->discount_data = (object) [
			'discount'        => $discount,
			'discount_amount' => $this->coupon->discount,
			'discount_type'   => $this->coupon->discount_type,
			'type'            => 'coupon',
			'coupon_code'     => $this->coupon_code,
		];
	}

	private function resetDiscounts($products)
	{
		foreach ($products as &$product) {
			unset($product->discount_data);
		}
	}

	private function getTotalOrder($products)
	{
		$total = 0;

		foreach ($products as &$product) {
			$total += ($product->price * $product->quantity);
		}

		return $total;
	}

	private function insertFullOrderDiscountRow($products)
	{
    $cart = new Cart;
		$cart->removeCouponsFromOrder();

		$total_price = $this->getTotalOrder($products);
		$discount    = $this->discount->getDiscount($total_price, $this->coupon->discount, $this->coupon->discount_type);

		$db = Factory::getDbo();

		$values = [
			'ordercode'       => $this->ordercode,
			'discount_group'  => $db->quote('coupon'),
			'discount_amount' => $db->quote($this->coupon->discount),
			'discount_type'   => $db->quote($this->coupon->discount_type),
			'coupon_code'     => $db->quote($this->coupon_code),
			'quantity'        => 1,
			'discount'        => $db->quote($discount),
			'product_id'      => 0,
		];

		$query = $db->getQuery(true)
			->insert($db->quoteName('#__larts_subs_cart_items'))
			->columns($db->quoteName(array_keys($values)))
			->values(implode(',', $values));

		$db->setQuery($query);
		$db->execute();

		return true;
	}

//  public function removeFullOrderDiscount()
//	{
//		$db = Factory::getDbo();
//		$query = $db->getQuery(true)
//			->delete($db->quoteName('#__larts_subs_cart_items'))
//			->where([
//				$db->quoteName('product_id') . ' = 0',
//				$db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
//			]);
//
//		$db->setQuery($query);
//
//		return $db->execute();
//	}

	public function check($coupon_code = '')
	{
		$this->coupon_code = $this->getCouponCode($coupon_code);
		$this->coupon = $this->getCoupon();

    $cart = new Cart;

		if(!self::checkCoupon()) {
			$cart->removeCouponsFromOrder();
			return false;
		}

		// All seems ok, so add the coupon to the order
		$cart->updateCouponCode($this->coupon_code);
    $this->message = Text::sprintf('COM_LARTSSUBS_COUPON_HAS_BEEN_APPLIED', $this->coupon_code);

		return true;
	}

	private function checkCoupon()
	{
		if (empty($this->coupon->id)) {
			$this->message = Text::sprintf('COM_LARTSSUBS_COUPON_INVALID', $this->coupon_code);
			return false;
		}

		// reached the limit?
		if ($this->coupon->limit > 0 && $this->coupon->limit <= $this->coupon->used) {
			$this->message = Text::sprintf('COM_LARTSSUBS_COUPON_LIMIT_REACHED', $this->coupon_code);
			return false;
		}

		$date = new Date;
		$now  = strtotime($date->format('Y-m-d'));

		$valid_from = $this->coupon->valid_from && $this->coupon->valid_from != '0000-00-00' ? strtotime($this->coupon->valid_from) : 0;
		$valid_to   = $this->coupon->valid_to && $this->coupon->valid_to != '0000-00-00' ? strtotime($this->coupon->valid_to) : 0;

		// Check if the coupon has a valid_from date and it is not in the future
		if ($valid_from && $valid_from > $now) {
			$this->message = Text::sprintf('COM_LARTSSUBS_COUPON_INVALID', $this->coupon_code);
			return false;
		}

		// Check if the coupon is expired
		if ($valid_to && $valid_to < $now) {
			$this->message = Text::sprintf('COM_LARTSSUBS_COUPON_EXPIRED', $this->coupon_code);
			return false;
		}

		// Check if the coupon is assigned to a user and that user is the current user
		if(!$this->passUserCheck()) {
			$this->message = Text::sprintf('COM_LARTSSUBS_COUPON_INVALID', $this->coupon_code);
			return false;
		}

		// Check if the coupon is assigned to specific products and the order contains one or more of these products
		if(!$this->passProductCheck()) {
			$this->message = Text::sprintf('COM_LARTSSUBS_COUPON_INVALID', $this->coupon_code);
			return false;
		}

		return true;
	}



	public function getCouponCode($coupon_code = '')
	{
		$this->coupon_code = $coupon_code ?: $this->coupon_code;

		if(!empty($this->coupon_code)) {
			return $this->coupon_code;
		}

		if (empty($this->ordercode)) {
			return '';
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select('coupon_code')
			->from($db->quoteName('#__larts_subs_cart_items'))
			->where($db->quoteName('ordercode') . ' = ' . (int) $this->ordercode)
			->where($db->quoteName('coupon_code') . ' != ' . $db->quote(''));

		$db->setQuery($query);
		$this->coupon_code = $db->loadResult();

		return $this->coupon_code;
	}

	public function getCoupon($coupon_code = '')
	{
		$this->coupon_code = $this->getCouponCode($coupon_code);

		if (empty($this->coupon_code)) {
			return (object) [];
		}

		if(!is_null($this->coupon)) {
			return $this->coupon;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select('*')
			->from($db->quoteName('#__larts_subs_coupons'))
			->where($db->quoteName('coupon_code') . ' = ' . $db->quote($this->coupon_code))
			->where($db->quoteName('published') . ' = 1')
			->order($db->quoteName('id') . ' ASC');

		$db->setQuery($query);

		$coupons = $db->loadObjectList();

		if (empty($coupons)) {
			$this->coupon = (object) [];
			return $this->coupon;
		}

		if (count($coupons) < 2) {
			$this->coupon = $coupons['0'];
			return $this->coupon;
		}

		$ordered_coupons = [];

		foreach ($coupons as $coupon)
		{
			$discount = $this->getCouponDiscount($coupon);
			$ordered_coupons[$discount] = $coupon;
		}

		krsort($ordered_coupons);

		$this->coupon = array_shift($ordered_coupons);

		return $this->coupon;
	}

	/**
	 * Check the coupon code
	 */
	private function getCouponDiscount($coupon)
	{
		if (empty($coupon->id)) {
			return 0;
		}

		// Check if we have reached the limit already.
		if ($coupon->limit > 0 && $coupon->limit <= $coupon->used)
		{
			return 0;
		}

		$date = new Date;
		$now  = strtotime($date->format('Y-m-d'));

		$valid_from = $coupon->valid_from && $coupon->valid_from != '0000-00-00' ? strtotime($coupon->valid_from) : 0;
		$valid_to   = $coupon->valid_to && $coupon->valid_to != '0000-00-00' ? strtotime($coupon->valid_to) : 0;

		// Check if the coupon has a valid_from date and it is not in the future
		if ($valid_from && $valid_from > $now)
		{
			return 0;
		}

		// Check if the coupon is expired
		if ($valid_to && $valid_to < $now)
		{
			return 0;
		}

		// Check if the coupon is assigned to a user and that user is the current user
		if(!$this->passUserCheck())
		{
			return 0;
		}

		// Check if the coupon is assigned to specific products and the order contains one or more of these products
		if(!$this->passProductCheck())
		{
			return 0;
		}

		$product_ids = $this->getCartProductIds();

		if ($coupon->full_order || empty($coupon->product_ids))
		{
			$total_price = 0;
			foreach ($product_ids as $product_id)
			{
				$products    = $this->getCartProductsByProductId($product_id);
				$total_price += $this->getTotalOrder($products);
			}

			return $this->discount->getDiscount($total_price, $coupon->discount, $coupon->discount_type);
		}

		$coupon_discount = 0;
		$coupon_products = explode(',', $coupon->product_ids);
		foreach ($product_ids as $product_id)
		{
			if(!in_array($product_id, $coupon_products))
			{
				continue;
			}

			$products = $this->getCartProductsByProductId($product_id);

			foreach ($products as $product)
			{
				$coupon_discount += $this->discount->getDiscount($product->price, $coupon->discount, $coupon->discount_type);
			}
		}

		return $coupon_discount;
	}

	/**
	 * Check if this coupon is assigned to one single customer.
	 */
	private function passUserCheck()
	{
		if (empty($this->coupon->userid))
		{
			return true;
		}

		return Factory::getApplication()->getIdentity()->id == $this->coupon->userid;
	}

	/**
	 * Check if there are any products in cart already.
	 */
	private function passProductCheck()
	{
		if (empty($this->coupon->product_ids))
		{
			return true;
		}

		$cart_ids    = $this->getCartProductIds();
		$product_ids = explode(',', $this->coupon->product_ids);

		$ids = array_intersect($product_ids, $cart_ids);

		return ! empty($ids);
	}

	/**
	 * Get all products that are currently in the cart for this session.
	 */
	public function getCartProductIds()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select('product_id')
			->from($db->quoteName('#__larts_subs_cart_items'))
			->where($db->quoteName('ordercode') . ' = ' . $this->ordercode)
			->where($db->quoteName('product_id') . ' > 0')
			->group($db->quoteName('product_id'));

		$db->setQuery($query);

		return $db->loadColumn();
	}

	public function getCartProductsByProductId($id)
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->select([
				'o.*',
				'p.name', 'p.price', 'p.period', 'p.period_type',
			])
			->from($db->quoteName('#__larts_subs_cart_items', 'o'))
			->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('o.product_id') . ' = ' . $db->quoteName('p.id') . ')')
			->where($db->quoteName('ordercode') . ' = ' . $this->ordercode)
			->where($db->quoteName('o.product_id') . ' = ' . (int) $id);

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	/**
	 * Update the coupon usage amount current amount + 1
	 */
	public function updateCouponUsage($couponcode = '')
	{
		$coupon = $this->getCoupon($couponcode);

		// Return if coupon variable is empty
		if (empty($coupon) || empty($coupon->id))
		{
			return true;
		}

		// Coupon has been counted already.
		//if (Factory::getSession()->get('coupon_code_used_' . $coupon->id))
		//{
		//	return true;
		//}

		Factory::getSession()->set('coupon_code_used_' . $coupon->id, true);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->update($db->quoteName('#__larts_subs_coupons'))
			->set($db->quoteName('used') . ' = ' . $db->quoteName('used') . ' + 1')
			->where($db->quoteName('id') . ' = ' . $coupon->id);

		$db->setQuery($query);

		return $db->execute();
	}

	/**
	 * Get a message from this class.
	 */
	public function getMessage()
	{
		return $this->message;
	}

  public function CreateRenewalCoupon($userid, $product_id, $discount, $namePrefix = 'Auto-Coupon')
  {
    $customers = new Customer; 
    $cust = $customers->loadCustomer($userid);
    $dNow = new Date();
    $now = $dNow->format('Y-m-d');
    $coupon_code = $this->GenerateCouponCode();
    
		$values = [
      'limit' => 1,
      'coupon_code' => $coupon_code,
      'name' => $namePrefix .' '. $cust->firstname .", ". $cust->name,
      'valid_from' => $now,
      'valid_to' => $now,
      'created_by' => 0,
      'discount' => $discount,
      'discount_type' => 'percent',
      'userid' => $userid,
      'type' => 0,
      'full_order' => 0,
      'product_ids' => $product_id,
      'published' => true,
		];

    if(!$this->storeCoupon($values)) {
      $coupon_code = '';
    }

    return $coupon_code;
    
  }
  
  public function CreateCoupon($userid, $namePrefix = 'Auto-Coupon')
  {
    $customers = new Customer; 
    $cust = $customers->loadCustomer($userid);
    $coupon_code = $this->GenerateCouponCode();
    
		$values = [
      'limit' => 1,
      'coupon_code' => $coupon_code,
      'name' => $namePrefix .' '. $cust->firstname .", ". $cust->lastname,
      //'valid_from' => 0,
      //'valid_to' => 0,
      'created_by' => 0,
      'discount' => 50,
      'discount_type' => 'percent',
      'userid' => 0,
      'type' => 0,
      'full_order' => 1,
      'product_ids' => '',
      'published' => true,
		];

    if(!$this->storeCoupon($values)) {
      $coupon_code = '';
    }

    return $coupon_code;
  }

  public static function storeCoupon($values)
  {
		$values = Database::validateFields('#__larts_subs_coupons', $values);
		
		if(isset($values['id']) && $values['id'] != 0) {
			return Database::update('#__larts_subs_coupons', 'id', $values);
		}
		else {
			return Database::insert('#__larts_subs_coupons', $values);
		}
	
		
  }
    
  public function GenerateCouponCode()
  {
    $length = 10; // Maximum of 15

    $chars = [
      str_split('BCDFHJKMN', 1),
      str_split('PQRTVWXZA', 1),
      str_split('34789', 1)
    ];

    $codes = [];
    while (count($codes) < $length)
    {
      $group = rand(0, 2);
      $max = count($chars[$group])-1;
      $pos = rand(0, count($chars[$group])-1);
      if($chars[$group][$pos] == '') {
        continue;
      }
      array_push($codes, $chars[$group][$pos]);
      $chars[$group][$pos] = '';
    }
    return implode($codes);
  }
  
	// Delete all expired / used
	public function removeExpiredCoupons()
	{
		$date = new Date;
		$date = $date->format('Y-m-d');

		$db = Factory::getDbo();
    $query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_coupons'))
		            ->where($db->quoteName('valid_to') . ' != NULL')
		            ->andWhere($db->quoteName('valid_to') . ' < ' . $db->quote($date));

		$db->setQuery($query);
		$db->execute();
  }
  
	// Delete all automatic coupons
	public function removeOldAutoCoupons()
	{
		$date = new Date;
		$date = $date->format('Y-m-d');

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_coupons'))
		            ->where($db->quoteName('type') . ' = ' . self::TYPE_AUTO)
		            ->andWhere($db->quoteName('valid_to') . ' != NULL')
		            ->andWhere($db->quoteName('valid_to') . ' < ' . $db->quote($date));

		$db->setQuery($query);
		$data = $db->loadObjectList();

    if(empty($data)) {
      return;
    }
    
		// For the statistics:
		$this->coupons_disabled = count($data);

		// Preparing the date:
		$date = new Date('-1 day');
		$date = $date->format('Y-m-d');

		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_coupons'))
		            ->where($db->quoteName('type') . ' = ' . self::TYPE_AUTO)
		            ->andWhere($db->quoteName('valid_to') . ' < ' . $db->quote($date));

		$db->setQuery($query);
		$db->execute();

		return true;
	}
  
}
