<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Coupon;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Renewal;

class Discount
{
	var $config      = null;
	var $ordercode   = null;
	var $coupon_code = null;

	public function __construct($ordercode = 0, $coupon_code = '')
	{
		$this->config = LartsConfig::get();
		$this->ordercode = $ordercode ?: intval(Factory::getSession()->get('ordercode'));
		$this->coupon_code = $coupon_code ?: '';
	}

	public function apply(&$orders)
	{
		$coupon = new Coupon($this->ordercode);

		$coupon_code = $this->coupon_code ?: $coupon->getCouponCode();

		$this->resetDiscounts($orders);

		if ($this->config->renewal_discount)
		{
			if (Factory::getApplication()->getIdentity()->id)
			{
				$discount_class = new Renewal($this->ordercode);
				$discount_class->apply($orders);
			}
		}

		if(!empty($coupon_code)) {
			$coupon->apply($orders, $coupon_code);
		}

		PluginManager::execPlugins('lartssubscriptions', 'onBeforeUpdateOrderDiscounts', $orders);

		$this->updateOrderDiscounts($orders);
	}

	public function resetDiscounts(&$orders)
	{
		$this->removeFullOrderDiscount();

		foreach ($orders as &$order)
		{
			$order->discount_group  = '';
			$order->discount_type   = '';
			$order->discount_amount = 0;
			$order->coupon_code     = '';
			$order->net_price       = $order->price;
			$order->discount        = 0;
		}
	}

	public function updateOrderDiscounts($orders)
	{
		foreach ($orders as &$order)
		{
			if (isset($order->discount_data))
			{
				$order->discount_group  = $order->discount_data->type;
				$order->discount_type   = $order->discount_data->discount_type;
				$order->discount_amount = $order->discount_data->discount_amount;
				$order->coupon_code     = isset($order->discount_data->coupon_code) ? $order->discount_data->coupon_code : '';
				$order->net_price       = $order->price - $order->discount_data->discount;
				$order->discount        = $order->discount_data->discount;

				unset($order->discount_data);
			}

			$this->updateOrderItemDiscount($order);
		}
	}

	private function updateOrderItemDiscount($order)
	{
		$db = Factory::getDbo();

		// Now set the new discount
		$query = $db->getQuery(true)
			->update($db->quoteName('#__larts_subs_cart_items'))
			->set([
				$db->quoteName('discount_group') . ' = ' . $db->quote($order->discount_group),
				$db->quoteName('discount_type') . ' = ' . $db->quote($order->discount_type),
				$db->quoteName('discount_amount') . ' = ' . $db->quote($order->discount_amount),
				$db->quoteName('coupon_code') . ' = ' . $db->quote($order->coupon_code),
				$db->quoteName('price') . ' = ' . $db->quote($order->price),
				$db->quoteName('discount') . ' = ' . $db->quote($order->discount),
			])
			->where([
				$db->quoteName('id') . ' = ' . $db->quote($order->id),
				$db->quoteName('product_id') . ' != 0',
			]);

		$db->setQuery($query);

		return $db->execute();
	}

	private function removeFullOrderDiscount()
	{
    // full order discount is a chart item with product_id = 0 and coupon code
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
			->delete($db->quoteName('#__larts_subs_cart_items'))
			->where([
				$db->quoteName('product_id') . ' = 0',
				$db->quoteName('ordercode') . ' = ' . (int) $this->ordercode,
			]);

		$db->setQuery($query);

		return $db->execute();
	}

	public function getDiscount($price, $discount, $type = 'percent')
	{
		// Coupon type is a normal price value
		if ($type == 'amount') {
			return $discount > $price ? $price : $discount;
		}

		// Coupon type is percentage
		$discounted_price = round(($price / 100) * $discount, 2);
		return $discounted_price > $price ? $price : $discounted_price;
	}

	public function getUserProductIds($use_grace_period = false)
	{
		$db = Factory::getDbo();

		$now = new Date;
		$now = $now->toSql();

		$query = $db->getQuery(true)
			->select('product_id')
			->from($db->quoteName('#__larts_subs_periods'))
			->where($db->quoteName('userid') . ' = ' . (int) Factory::getApplication()->getIdentity()->id)
			->where($db->quoteName('valid_to') . ' >= ' . $db->quote($now))
			->where($db->quoteName('product_id') . ' != 0')
			->group($db->quoteName('product_id'));

		$db->setQuery($query);
		$ids = $db->loadColumn();

		if(!$use_grace_period || ! $this->config->expiration_grace_period)
		{
			return $ids;
		}

		$grace = new Date('- ' . (int) $this->config->expiration_grace_period . 'DAYS');
		$grace = $grace->toSql();

		$query = $db->getQuery(true)
			->select('product_id')
			->from($db->quoteName('#__larts_subs_periods'))
			->where($db->quoteName('userid') . ' = ' . (int) Factory::getApplication()->getIdentity()->id)
			->where($db->quoteName('status') . ' IN (0, 1)')
			->where($db->quoteName('valid_to') . ' < ' . $db->quote($now))
			->where($db->quoteName('valid_to') . ' >= ' . $db->quote($grace))
			->where($db->quoteName('product_id') . ' != 0')
			->group($db->quoteName('product_id'));

		$db->setQuery($query);

		return array_merge($ids, $db->loadColumn());
	}
}
