<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\GeoIP;
defined('_JEXEC') or die;

require_once dirname(__DIR__) . '/../geoip2/autoload.php';

use Joomla\CMS\Factory;
use Joomla\CMS\Http\HttpFactory;

use GeoIp2\Database\Reader;
use GeoIp2\Model\City;

use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\GeoIP\GeoIPDB;
use LartsSubscriptions\Cronjob;

class GeoIP
{
	var $ip      = '';
	var $reader  = null;
	var $records = [];
	var $data    = [];
	var $message = '';

  public function __construct($ip = '')
	{
		$this->ip = $ip ?: $this->getIP();
	}

  public function UpdateMaxmindDB()
  {
    $updater = new GeoIPDB;

    $key = LartsConfig::get('maxmind_key');
    $result = $updater->update($key, 'Country');

    $bRet = true;
    $this->message = '';
    if ($result->state == 'error') {
      $this->message = 'Error: ' .$result->message;
      $bRet = false;
    }
    else if ($result->message) {
      $this->message = 'Info: ' . $result->message;
    }

    if(!$last_date = $updater->getVersion()) {
      $this->message = 'Error: Invalid version';
      $bRet = false;
    }

    return $bRet;
  }
  
  public static function getDatabasePath()
	{
		return JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/lib/geoip2/GeoLite2-Country.mmdb';
	}

	public static function hasDatabase()
	{
		$file = self::getDatabasePath();

		return file_exists($file)
			&& filesize($file) > 10;
	}

	public static function getIP()
	{
    $ip = '';
		if (!empty($_SERVER['HTTP_X_FORWARDED_FOR']) && self::isValidIP($_SERVER['HTTP_X_FORWARDED_FOR']))
		{
			$ip =  $_SERVER['HTTP_X_FORWARDED_FOR'];
		}
		else if (!empty($_SERVER['HTTP_X_REAL_IP']) && self::isValidIP($_SERVER['HTTP_X_REAL_IP']))
		{
			$ip =  $_SERVER['HTTP_X_REAL_IP'];
		}
		else if (!empty($_SERVER['HTTP_CLIENT_IP']) && self::isValidIP($_SERVER['HTTP_CLIENT_IP']))
		{
			$ip = $_SERVER['HTTP_CLIENT_IP'];
		}
    else {
  		$ip =  $_SERVER['REMOTE_ADDR'];
    }
    
		// Clear IP addresses for possible local IP address list.
		if (in_array($ip, ['127.0.0.1', '::1']) || substr($ip, 0, 7) == '192.168')
		{
			$ip = '';
		}
    return $ip;
	}

  private static function isValidIP($string)
	{
		return filter_var(
			$string,
			FILTER_VALIDATE_IP,
			FILTER_FLAG_IPV4
		//	| FILTER_FLAG_IPV6
			| FILTER_FLAG_NO_PRIV_RANGE
			| FILTER_FLAG_NO_RES_RANGE
		);
	}
  
	public function getReader()
	{
		if(!is_null($this->reader)) {
			return $this->reader;
		}

		if(!self::hasDatabase()) {
			return false;
		}
		$this->reader = new Reader(self::getDatabasePath());

		return $this->reader;
	}

  public function lookupIP($ip = null)
	{
		if (!empty($ip)) {
			$this->ip = $ip;
		}
    
		if (empty($this->ip)) {
			return false;
		}

		if (isset($this->records[$this->ip])) {
			return $this->records[$this->ip];
		}

		if(!$reader = $this->getReader()) {
			return false;
		}

		try
		{
			$this->records[$this->ip] = $reader->country($this->ip);
		}
		catch (\Exception $e)
		{
			$this->records[$this->ip] = false;
		}

		return $this->records[$this->ip];
	}

  
	public function getCountryByIP()
	{

    $use = LartsConfig::get('use_geoip');
    if(empty($use)) {
      return '';
    }

    $ip = GeoIP::getIP();
    if(empty($ip)) {
      return '';
    }

    if($use == 'ipstack') {
      return $this->getCountryByIPStackApi($ip);
    }
    else if($use == 'maxmind') {
      $record = $this->lookupIP($ip);
      if(!empty($record)) {
        return $record->country->isoCode;
      }
    }
    return '';
	}

   private function getCountryByIPStackApi($ip)
	{
		$session = Factory::getSession();

		// Loading access token for ipstack
		$access_token = trim(LartsConfig::get('ipstack_token'));

		// Getting the country code from the session.
		$country_code = $session->get('country_code', null);

		// Stop when the token is empty
		if (empty($access_token)) {
			return null;
		}

		if(!empty($country_code)) {
			return $country_code;
		}

		try
		{
			// Request the Country code by the given IP.
			$package = HttpFactory::getHttp()->get('http://api.ipstack.com/' . $ip . '?access_key=' . $access_token, null, 15);
		}
		catch (\RuntimeException $e) {
			return '';
		}

		// When the check goes wrong.
		if(!$package || $package->code != 200 || empty($package->body))
		{
			return '';
		}

		// Decode the JSON string
		$results = json_decode($package->body);

		// Setting the country code in the session
		$session->set('country_code', $results->country_code);

		return $results->country_code;
	}
}