<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\GeoIP;
defined('_JEXEC') or die;

require_once dirname(__DIR__) . '/../php-archive/autoload.php';
use splitbrain\PHPArchive\Tar;

use Joomla\CMS\Factory;
use Joomla\CMS\Http\HttpFactory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filesystem\File;
use Joomla\Filesystem\Folder;

//require_once JPATH_LIBRARIES . '/geoip/autoload.php';

//jimport('joomla.filesystem.file');
//jimport('joomla.filesystem.folder');

class GeoIPDB
{
	private $database_name;
	private $database_file;
	private $temp_folder;
	private $package_file;
	private $date_file;
	private $license_key;
	private $force;

  public function __construct()
	{
    $database_file = '';
  }
  
	public function update($key, $name = 'City')
	{
		$this->database_name = 'GeoLite2-' . $name;
		$this->database_file = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/lib/geoip2/' . $this->database_name . '.mmdb';
		$this->temp_folder   = Factory::getConfig()->get('tmp_path');
		$this->package_file  = $this->temp_folder . '/' . $this->database_name . '.tar.gz';
		$this->date_file     = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/lib/geoip2/' . $this->database_name . '.date.txt';
		$this->license_key   = $key;
		$this->force         = true;

		if ( ! $this->license_key) {
			return $this->error(Text::_('GEO_MESSAGE_ERROR_NO_LICENSE_KEY'));
		}

		$result = $this->download();

		if ($result->state == 'error') {
			return $result;
		}

		$this->deleteTempFiles();

		return $result;
	}

	public function deleteTempFiles()
	{
		File::delete($this->package_file);
	}

	public function getVersion()
	{
		if (!is_file($this->date_file)) {
			return 0;
		}
		return file_get_contents($this->date_file);
	}

	private function download()
	{
		$last_date = $this->getVersion();

		if ( ! $this->force
			&& $last_date && date('Y-m') == date('Y-m', $last_date)
			&& GeoIP::hasDatabase())
		{
			return $this->success(Text::_('GEO_MESSAGE_UPTODATE'));
		}

		$url = 'https://download.maxmind.com/app/geoip_download'
			. '?edition_id=' . $this->database_name
			. '&suffix=tar.gz'
			. '&license_key=' . $this->license_key;

    // Set up custom headers for a single request.
    $headers = array('Accept' => 'application/gzip');
		$package = HttpFactory::getHttp()->get($url, $headers, 30);

		if ( empty($package) || $package->getStatusCode() != 200) {
			return $this->error(Text::_('GEO_MESSAGE_ERROR_UPDATE'));
		}

    $last_modified = $package->getHeaderLine('last-modified');

		if ( ! $this->force
			&& $last_date
			&& strtotime($last_modified) <= $last_date
			&& GeoIP::hasDatabase()
		)
		{
			return $this->success(Text::_('GEO_MESSAGE_UPTODATE'));
		}

		File::write($this->package_file, $package->body);

		$result = $this->unpack();

		if ($result->state == 'error')
		{
			return $result;
		}

		File::write($this->date_file, strtotime($last_modified));

		return $this->success();
	}

	private function success($message = '')
	{
		return (object) [
			'state'   => 'success',
			'message' => $message,
		];
	}

	private function error($message = '')
	{
		return (object) [
			'state'   => 'error',
			'message' => $message,
		];
	}

	private function unpack()
	{
		try
		{
			$tar = new Tar;
			$tar->open($this->package_file);
			$files = $tar->contents();

			$database_file = '';
			foreach ($files as $file)
			{
				if (strpos($file->getPath(), '.mmdb') === false)
				{
					continue;
				}

				$database_file = $file->getPath();
				break;
			}

			$tar = new Tar;
			$tar->open($this->package_file);
			$tar->extract($this->temp_folder);

			rename($this->temp_folder . '/' . $database_file, $this->database_file);

			Folder::delete(dirname($this->temp_folder . '/' . $database_file));
		}
		catch (\Exception $e)
		{
			return $this->error(Text::_('GEO_MESSAGE_ERROR_UNPACK'));
		}

		return $this->success();
	}
}
