<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
//use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Base\Database;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Cart;
use LartsSubscriptions\Order;
use LartsSubscriptions\Pdf\PdfPrinter;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Price;


class Invoice
{
  var $config = null;
	var $path = null;
  
  const TYPE_DEBIT = 0;    // db.invoice_type
  const TYPE_CREDIT = 1;   // db.invoice_type
  const TYPE_OFFER = 2;    // db.invoice_type
  const UNPAID = 0;        // db.paid
  const PAID = 1;          // db.paid
  
	function __construct()
	{
		$this->path = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/invoices/';
		$this->config = LartsConfig::get();
	}

  public function getPath() {
    return $this->path;
  }
  
  public function store($data, $paid = true)
	{
		$date   = new Date;
		$db     = Factory::getDbo();
		$amount = new OrderAmounts($data['ordercode'], false);

    $invoice_number = $this->generateInvoiceNumber();

    // Prepare invoice data
		$invoice_data = (object) [
      'invoice_number'  => $invoice_number,
			'userid'          => $data['userid'],
			'ordercode'       => (int) $data['ordercode'],
			'published'       => 1,
			'paid'            => ($paid ? self::PAID : self::UNPAID),
			'provider'        => $data['provider'],
			'net_price'       => round($amount->netprice(), 2),
			'gross_price'     => round($amount->grossprice(), 2),
			'vat_total'       => round($amount->vatprice(), 2),
			'disco_price'     => round($amount->discount(), 2),
			'invoicedate'     => $date->toSql(),
			'vat_percentage'  => $amount->getVatPercentsAppliedStr(),
			'transaction_id'  => $data['transaction_id'],
      'dlkey'           => md5($invoice_number),
      'vat_number'      => $data['vat_number'],
		];

		// Store the invoice information and generate an invoice number;
		Factory::getDbo()->insertObject('#__larts_subs_invoices', $invoice_data, 'id');
		return $invoice_data->id;

	}

  public static function update($record, $key)
  {
    $dbhlp = new Database;
    return $dbhlp->update('#__larts_subs_invoices', $key, $record);
  }

	// Generates an invoice number.
	// Incremental ID / With Prefix / Monthly Count + Prefix
	public function generateInvoiceNumber()
	{
    switch ($this->config->invoice_numbering) {
    case 2:
      $count = $this->getInvoiceCountForThisMonth();
      $invoice_number = date('Ym') . '-' . str_pad($count + 1, 4, '0', STR_PAD_LEFT);
      break;
    case 1:
    default:
      $invoice_number = $this->getHighestInvoiceNumber();
      $invoice_number++;
      break;
		}

		if ($this->config->use_invoice_prefix == 1) {
			$invoice_number = $this->config->invoice_prefix . $invoice_number;
		}

		return $invoice_number;
	}

	// Giving back the latest invoice_number from the database. From there we count up with one (1)
	private function getHighestInvoiceNumber()
	{
		$db = Factory::getDbo();

		// Get last added invoice_number
		$query = $db->getQuery(true)
		            ->select('MAX(id)')
		            ->from($db->quoteName('#__larts_subs_invoices'));
		$db->setQuery($query);
		$number = $db->loadResult();

		return max((int)$number, $this->config->invoice_number_start);
	}

	// Get the amount of created invoices for the current month.
	//With this number the invoice number can be generated.
	private function getInvoiceCountForThisMonth()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['COUNT(id) as total'])
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where('MONTH(invoicedate)' . ' = ' . date('m'))
		            ->where('YEAR(invoicedate)' . ' = ' . date('Y'));

		$db->setQuery($query);

		$result = $db->loadObject();

		return $result->total;
	}

  public function createInvoicePdf($id, $refund = false)
	{
		// load invoice from db
		$invoice = $this->load($id);
    if(empty($invoice)) {
      return false;
    }
    
    $variables = Order::getOrderData($invoice->ordercode, false);

		// Setting the payment state for the invoice.
		$payment_state = ($invoice->paid ? Text::_('COM_LARTSSUBS_INVOICE_STATE_PAID') : Text::_('COM_LARTSSUBS_INVOICE_STATE_UNPAID'));
		$invoice_number = ! empty($invoice->invoice_number) ? $invoice->invoice_number : $invoice->id;

		// Variables for the message:
		$variables2 = [
			'payment_state'         => $payment_state,
			'date'                  => $invoice->invoicedate,
			'userid'                => $invoice->userid,
			'invoice_id'            => $invoice_number,
			'transaction_id'        => $invoice->transaction_id,
			'payment_provider'      => $invoice->provider,
		];
    if($refund){
			$variables['invoice_refund'] = Price::_(-1 * $variables['invoice_netto']);
    }
    $variables = array_merge($variables, $variables2);

		$message = new Message;

		// Getting the invoice template.
		$footer = $message->alias('invoice-footer')
		                ->user($invoice->userid)
		                ->variables($variables)
		                ->getBody();

    // Getting the invoice template.
		$body = $message->alias($refund ? 'refund-template' : 'invoice-template')
		                ->user($invoice->userid)
		                ->variables($variables)
		                ->getBody();


    
		// Output the HTML content
    $pdf = new PdfPrinter;
    $pdf->setFooter($footer);
    $pdf->doPrint($body);

    $src = $pdf->getfile();
		// The new name for the invoice (destination for saving the file)
		$dest = $this->getPath() . $invoice_number . '.pdf';

		// Copy the file now.
		File::copy($src, $dest);
		File::delete($src);

		PluginManager::execPlugins('lartssubscriptions', 'onAfterCreateInvoice', $variables);

		return true;
	}

  public function setPublished($id, $bSet = true)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_invoices'))
		            ->set($db->quoteName('published') . ' = ' . ($bSet ? '1' : '0'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);

		return $db->execute();
  }

  public static function load($id)
	{
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);
		return $db->loadObject();
 }
  
  public function sendInvoice($id, $template = 'send-invoice')
	{
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['invoice_number', 'ordercode', 'userid', 'provider'])
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);
		$db->setQuery($query);
		$inv = $db->loadObject();

		$attachment = $this->getPath() . $inv->invoice_number . '.pdf';
		if(!file_exists($attachment)) {
			return false;
		}

    $cart = new Cart($inv->ordercode);
    $variables = [
			'invoice_id'        => $id,
			'invoice_number'    => $inv->invoice_number,
			'user_id'           => $inv->userid,
			'invoice_file'      => $attachment,
      'orderlist'         => $cart->getOrderedItemsHTML(false),
      'orderlist_simple'  => $cart->getOrderedItemsHTML(true),
			'ordercode'         => $inv->ordercode,
    ];

		Message::getInstance($template)
		             ->variables($variables)
		             ->invoice($id)
		             ->attachment($attachment)
		             ->send();

    $this->setPublished($id, true);

		// Trigger plugin to make actions after sending an invoice
    $data = [
			'invoice_id'     => $id,
			'invoice_number' => $inv->invoice_number,
			'user_id'        => $inv->userid,
			'invoice_file'   => $attachment,
		];
		PluginManager::execPlugins('lartssubscriptions', 'onAfterSendInvoice', $data);

		// Remove file if set in the config.
		if(!LartsConfig::get('store_invoice_locally')) {
			File::delete($attachment);
		}

		return true;
	}

	public function getInvoiceById($id)
	{
		if(!$id) {
			return [];
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);

		return $db->loadObject();
	}

	public static function getInvoiceByOrdercode($ordercode = null)
	{
		if(!$ordercode) {
			return [];
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);
		return $db->loadObject();
	}

  public static function getInvoiceByDownloadKey($key)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName("#__larts_subs_invoices"))
		            ->where($db->quoteName("dlkey"). " = '" .$key. "'");
		$db->setQuery($query);
		return $db->loadResult();
	}

  public function removeInvoicePDFFiles($invoice_numbers)
	{
		foreach ($invoice_numbers as $inumber)
		{
      $file = $this->getPath() . $inumber . '.pdf';
			if (file_exists($file)) {
				unlink($file);
			}
		}
	}

  public function removeByOrdercode($ordercodes = [])
	{
		if (empty($ordercodes)) {
			return true;
		}

    // remove invoice pdfs
    $invoice_numbers = array();
    foreach ($ordercodes as $ordercode) {
      $invoice = $this->getInvoiceByOrdercode($ordercode);
      $invoice_numbers[] = $invoice->invoice_number;
    }
    $this->removeInvoicePDFFiles($invoice_numbers);
    
    $db = Factory::getDbo();
    $query = $db->getQuery(true)
                ->delete($db->quoteName('#__larts_subs_invoices'))
                ->where($db->quoteName('ordercode') . ' IN ' . '( ' . implode(',', $ordercodes) . ' )');
    $db->setQuery($query);
    return $db->execute();
	}
  
  // set paid status
  public static function setPaid($ordercode, $paid)
  {
		$db = Factory::getDbo();
		$query = $db->getQuery(true);
		$query->update($db->quoteName('#__larts_subs_invoices'))
		      ->set($db->quoteName('paid') . ' = '. $paid)
		      ->where($db->quoteName('ordercode') . ' = ' . $ordercode);

		$db->setQuery($query);

		return $db->execute();
  }

}