<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

/*
 * Product can be
 * - File
 * - Subscription itself (e.g. become member of an internet group)
 * - Ticket (e.g. named ticket for concert)
 * - Movie stream (not downloadable)
 * - analog product (needs shipping)
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

use LartsSubscriptions\Categories;
use LartsSubscriptions\Files;
use LartsSubscriptions\Base\Image;
use LartsSubscriptions\ThirdParty\LartsPkgManager;
use LartsSubscriptions\VAT\VATClasses;
use LartsSubscriptions\Base\Redirect;

class Product
{
  const UNKNOWN = -1;
  const FILE = 0;
  const SUBSCRIPTION = 1;
  const TICKET = 2;
  const MOVIE_STREAM = 3;
  const ANALOG = 4;
  const CATEGORY = 5;
  const GROUP_ACCESS = 6;
  const _LAST_ = 7;

  public static function GetTypeStr($type) {
    switch($type) {
    case self::FILE:
      return Text::_('COM_LARTSSUBS_FILE');
    case self::SUBSCRIPTION:
      return Text::_('COM_LARTSSUBS_SUBSCRIPTION');
    case self::TICKET:
      return Text::_('COM_LARTSSUBS_TICKET');
    case self::MOVIE_STREAM:
      return Text::_('COM_LARTSSUBS_MOVIE_STREAM');
    case self::ANALOG:
      return Text::_('COM_LARTSSUBS_ANALOG');
    case self::CATEGORY:
      return Text::_('COM_LARTSSUBS_CATEGORY');
    case self::GROUP_ACCESS:
      return Text::_('COM_LARTSSUBS_GROUP_ACCESS');
    }
    return Text::_('COM_LARTSSUBS_ERROR');
  }
  
  public static function getTypeOptions()
  {
    $options = array();
    for($i = 0; $i < self::_LAST_; ++$i) {
      $options[$i] = self::GetTypeStr($i);
    }
    return $options;
  }
  
  public static function getRefId($product_id)
  {
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('product_ref')
			->from($db->quoteName('#__larts_subs_products'))
			->where($db->quoteName('id') . ' = ' . (int) $product_id);
		$db->setQuery($query);
		$result = $db->loadResult();

    return (!empty($result) ? $result : $product_id);
 }

  public static function getProduct($product_id, $select = '*')
  {
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select($select)
			->from($db->quoteName('#__larts_subs_products'))
			->where($db->quoteName('id') . ' = ' . (int) $product_id);

		$db->setQuery($query);
		return $db->loadObject();
  }

  public static function getDownloadLinkByProductId($product_id)
  {
    $prod = self::getProduct($product_id);
    return self::getDownloadLink($prod);
  }

  public static function getDownloadLink($product, &$file)
  {
    $user = Factory::getApplication()->getIdentity();
    if(empty($product || $user->id == 0))
      return '';
    
    $link = '';
    
    if(!empty($product->downloads_menu_item))
    {
      $link = Redirect::getLinkFromMenuId($product->downloads_menu_item);
      return $link;
    }
    
    if (isset($product->content_url) && strlen($product->content_url) > 0)
    {
      // check if local or external url
      // e.g.: 'index?option....' ->local
      //       'http://...' ->external
      $local = true;
      $arr = parse_url($product->content_url);
      if(isset($arr["scheme"]) || isset($arr["host"])) {
        $local = false;
      }

      $link = $local ? Route::_($product->content_url) : $product->content_url;
      return $link;
    }
            
    switch($product->type) {
    case self::FILE:
      // $product->source_id points to file
      $file = Files::lookup($product->source_id);
      if($file != null) {
        $link = 'index.php?option=com_lartssubscriptions&task=download.file&key=' .$file->hash;
      }
      break;
    case self::SUBSCRIPTION:
      // we have a download, if linked with lartspkgemanaer
      if((int)$product->pkgmanager_id > 0) {
        if(LartsPkgManager::IsInstalled()) {
          $api = LartsPkgManager::getAPI();
          $key = $api->getUpdateKey($user->id, $product->pkgmanager_id, true);
          //$link = $api->getDownloadLink($product->pkgmanager_id);
          
          if($key != null) {
            $ver = $api->getLastVersion($product->pkgmanager_id);
            if(!empty($ver)) {
              $file = new \StdClass;
              $file->version = $ver;
            }
            $link = 'index.php?option=com_lartssubscriptions&task=download.package&key=' .$key;
          }

        }
      }
      break;
    case self::TICKET:
      // $product->source_id points to template
      // since tickets have their own table, just link to the users tickets table
      $link = 'index.php?option=com_lartssubscriptions&view=mytickets';
      break;
    case self::MOVIE_STREAM:
      // $product->source_id points to file
      $file = Files::lookup($product->source_id);
      if($file != null) {
        $link = 'index.php?option=com_lartssubscriptions&task=download.stream&key=' .$file->hash;
      }
      break;
    case self::ANALOG:
    case self::GROUP_ACCESS:
      break;
    case self::CATEGORY:
      // $product->source_id points to category
      $cat = Categories::getHash($product->source_id);
      if($cat != null) {
        $link = 'index.php?option=com_lartssubscriptions&task=download.category&key=' .$cat->hash;
      }
      break;
   }
   return Route::_($link);
 }
 
  public static function removeImage($id)
  {
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_products'))
		            ->set($db->quoteName('image') . ' = null')
		            ->where($db->quoteName('id') . ' = ' .$id);
		$db->setQuery($query);
    //$sql = $query->__toString();
    
		return $db->execute();
  }

  public static function selectProductsOfCat($catid)
  {
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('prod.*')
                ->from($db->quoteName('#__larts_subs_product_categories', 'cat'))
                ->leftJoin($db->quoteName('#__larts_subs_products', 'prod') .' ON '. $db->quoteName('cat.product_id'). '=' . $db->quoteName('prod.id'))
		            ->where($db->quoteName('cat.category_id') . '=' .$catid)
		            ->andWhere($db->quoteName('published') . '!= 0')
            ;

    if(Factory::getApplication()->getIdentity()->id == 0) {
      // guest
			$query->where($db->quoteName('ispublic') . ' = 1');
    }
    
    $db->setQuery($query);
    //$sql = $query->__toString();
		$result = $db->loadObjectList();

		return $result;
  }
  
  public static function selectProductsNotCategoriesed()
  {
    //$sql = 'SELECT * FROM Table1 WHERE id NOT IN (SELECT id FROM Table2)';
 		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('prod.id, prod.*')
                ->from($db->quoteName('#__larts_subs_products', 'prod'))
		            ->where($db->quoteName('published') . '!= 0')
		            ->andWhere($db->quoteName('prod.id') . ' NOT IN (SELECT '.$db->quoteName('product_id'). ' FROM '.$db->quoteName('#__larts_subs_product_categories'). ')')
            ;

    if(Factory::getApplication()->getIdentity()->id == 0) {
      // guest
			$query->where($db->quoteName('ispublic') . ' = 1');
    }


    $db->setQuery($query);
    //$sql = $query->__toString();
		$result = $db->loadObjectList();
		return $result;
  }

  public static function prepare($products)
  {
		foreach ($products as &$product)
		{
      Product::validateImage($product);

      if($product->downloads_menu_item != 0) {
  			$product->menu_link = self::getMenuItemFromDatabase($product->downloads_menu_item);
			}
		}
  }
  
  public static function validateImage(&$product)
  {
    $bNoImage = empty($product->image);
    $bNoImage |= file_exists(JPATH_BASE. '/images/' .$product->image);
    if($bNoImage) {
      $product->image = Image::getNoImage();;
    }
  }
  
 	public static function getMenuItemFromDatabase($menu_id)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['link'])
		            ->from($db->quoteName('#__menu'))
		            ->where($db->quoteName('id') . ' = ' . $menu_id);
		$db->setQuery($query);
		$menu =  $db->loadObject();
		return !empty($menu->link) ? $menu->link : null;
	}

  public static function getVATRateByCountry($product, $country_code)
  {
    [$country, $vats] = VATClasses::getByCountry($country_code);
    $rate = 0;

    $vatclass = $product->vat_class;
    if($vatclass == -1) {   // no VAT?
      if($country->force_vat_payment) {
        $vatclass = 0;    // fallback to std vat
      }
    }
      
    if($vatclass != -1) {   // no VAT?
      if(isset($vats[$vatclass])) {
        $rate = $vats[$vatclass];
      }
    }
    return $rate;
  }

  public static function getMaxItemId()
	{
 		$db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select('MAX(ordering)')
      ->from('#__larts_subs_products');
    $db->setQuery($query);
    return $db->loadResult();
	}

  
}
  
  
  

