<?php

/*
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;

use LartsSubscriptions\Order;
use LartsSubscriptions\Pdf\PdfPrinter;
use LartsSubscriptions\Config as LartsConfig;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Message;
use LartsSubscriptions\Product;
use LartsSubscriptions\OrderAmounts;


class Ticket {

  public function __construct() {
    
  }

  /** Generate an ticket number and store the ticket information and 
   * 	
   * @param type $data
   * @return type
   */
  public function store($data)
	{
		$date   = new Date;
		$db     = Factory::getDbo();
		$amount = new OrderAmounts($data['ordercode']);

		// call plugins to get ticket number, else create one 
    $ret = PluginManager::execPlugins('lartssubscriptions', 'onGenerateTicketNumber', $data, false, false);
    $ret = PluginManager::concatStrings($ret, '');
    if(!empty($ret)) {
      $ticket_number = $ret;
    }
    else {
      $ticket_number = $this->generateTicketNumber();
    }

    // Prepare ticket data
		$record = (object) [
      'ticket_number'   => $ticket_number,
			'userid'          => $data['userid'],
			'product_id'      => (int) $data['product_id'],
			'ordercode'       => (int) $data['ordercode'],
			'date'            => $date->toSql(),
			'published'       => 1,
      'dlkey'           => md5($data['userid'].'_'.$ticket_number),
		];

		Factory::getDbo()->insertObject('#__larts_subs_tickets', $record, 'id');
		return $record->id;
	}

  /** Load Ticket by id
   * 
   * @param type $id
   * @return type
   */
  public static function load($id)
	{
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('t.*, p.name, p.source_id, tmpl.alias, o.amounts')
		            ->from($db->quoteName('#__larts_subs_tickets', 't'))
                ->leftJoin($db->quoteName('#__larts_subs_products', 'p') . ' ON(' . $db->quoteName('t.product_id') . ' = ' . $db->quoteName('p.id') . ')')
                ->leftJoin($db->quoteName('#__larts_subs_templates', 'tmpl') . ' ON(' . $db->quoteName('p.source_id') . ' = ' . $db->quoteName('tmpl.id') . ')')
                ->leftJoin($db->quoteName('#__larts_subs_orders', 'o') . ' ON(' . $db->quoteName('t.ordercode') . ' = ' . $db->quoteName('o.ordercode') . ')')
		            ->where($db->quoteName('t.id') . ' = ' . (int) $id);

		$db->setQuery($query);
		return $db->loadObject();
  }

  /** Lookup for the ticket(s) by ordercode
   * 
   * @param type $ordercode
   * @return type
   */
  public function lookupOrdercode($ordercode)
	{
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName('#__larts_subs_tickets'))
		            ->where($db->quoteName('ordercode') . ' = ' . (int) $ordercode);

		$db->setQuery($query);
		return $db->loadObjectList();
  }

  public function setPublished($id, $bSet = true)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_tickets'))
		            ->set($db->quoteName('published') . ' = ' . ($bSet ? '1' : '0'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);

		$db->setQuery($query);

		return $db->execute();
  }

 	public function generateTicketNumber()
	{
		$config = LartsConfig::get();

    $ticket_number = $this->getHighestTicketNumber();
    $ticket_number++;

		if ($config->use_ticket_prefix == 1) {
			$ticket_number = $config->ticket_prefix . $ticket_number;
		}

		return $ticket_number;
	}

  private function getHighestTicketNumber()
	{
		$config = LartsConfig::get();

    $db = Factory::getDbo();

		// Get last added ticket_number
		$query = $db->getQuery(true)
		            ->select('MAX(id)')
		            ->from($db->quoteName('#__larts_subs_tickets'));
		$db->setQuery($query);
		$number = $db->loadResult();

		return max((int)$number, $config->ticket_number_start);
	}

  public function createTicketPdf($id)
	{
		// Required helpers to create this PDF ticket
		// Do NOT edit the required files! It will damage your component.

		// load ticket from db
		$ticket = $this->load($id);
    if(empty($ticket)) {
      return false;
    }
 		$ticket_number = !empty($ticket->ticket_number) ? $ticket->ticket_number : $ticket->id;

    // load product of ticket
    $product = Product::getProduct($ticket->product_id);
    if(empty($product)) {
      $err_msg = "Product not found!";
      return false;
    }
    
    // load invoice data
    $order = Order::lookup($ticket->ordercode);
    if(empty($order)) {
      $err_msg = "Order not found!";
      return false;
    }
    $amounts = new OrderAmounts();
    $amounts->setJson($order->amounts);
    
    $vat_percent = Product::getVATRateByCountry($product, $order->country_code);
    $vat_amount = round(($product->price / 100) * $vat_percent, 2);
    

		// Variables for the message:
		$variables = [
			'date'                  => $ticket->date,
			'userid'                => $ticket->userid,
			'ticket_id'             => $id,
			'ticket_number'         => $ticket_number,
			'ordercode'             => $ticket->ordercode,
			'ticket_brutto'         => $product->price + $vat_amount,
			'ticket_netto'          => $product->price,
			'ticket_vat'            => $vat_amount,
			'ticket_vat_pct'        => $vat_percent,
		];

 		PluginManager::execPlugins('lartssubscriptions', 'onPrintTicket', $variables);

		$message = new Message;

    // Getting the ticket template.
		$body = $message->alias($ticket->alias)
		                ->user($ticket->userid)
		                ->variables($variables)
		                ->getBody();
    
    $pdf = new PdfPrinter;
    $pdf->doPrint($body);
    $src = $pdf->getfile();

    // The new name for the ticket (destination for saving the file)
		$dest = $this->getFilepath($id);

		PluginManager::execPlugins('lartssubscriptions', 'onAfterCreateTicket', $variables);

		// Copy the file now.
		File::copy($src, $dest);
		File::delete($src);

		return true;
  }
  
  public function getPath()
  {
		$path = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/tickets';
    return $path;
  }

  public static function getFilepath($ticket_id)
  {
		$file = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/tickets/ticket_' . $ticket_id . '.pdf';
    return $file;
  }

  public function sendTicket($id, $template = 'send-ticket')
	{
    $db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select(['ticket_number', 'ordercode', 'userid'])
		            ->from($db->quoteName('#__larts_subs_tickets'))
		            ->where($db->quoteName('id') . ' = ' . (int) $id);
		$db->setQuery($query);
		$ticket = $db->loadObject();

		$attachment = $this->getFilepath($id);
		if(!file_exists($attachment)) {
			return false;
		}

    $variables = [
			'ticket_id'         => $id,
			'ticket_number'     => $ticket->ticket_number,
			'user_id'           => $ticket->userid,
			'ticket_file'       => $attachment,
			'ordercode'         => $ticket->ordercode,
    ];

		Message::getInstance($template)
		             ->variables($variables)
		             ->ticket($id)
		             ->attachment($attachment)
		             ->send();

    $this->setPublished($id, true);

		// Trigger plugin to make actions after sending an ticket
    $data = [
			'ticket_id'     => $id,
			'ticket_number' => $ticket->ticket_number,
			'user_id'       => $ticket->userid,
			'ticket_file'   => $attachment,
		];
		PluginManager::execPlugins('lartssubscriptions', 'onAfterSendTicket', $data);

		// Remove file if set in the config.
		if(!LartsConfig::get('store_ticket_locally')) {
			File::delete($attachment);
		}

		return true;
	}

  public static function getAllOfUser($userid, $sort = 't.product_id')
  {
    $db = Factory::getDbo();
    $query = $db->getQuery(true);
    $query->select('t.*, p.content_url, p.name, p.alias, p.type, p.source_id')
          ->from($db->quoteName('#__larts_subs_tickets', 't'))
          ->leftJoin($db->quoteName('#__larts_subs_products', 'p') . ' ON(' . $db->quoteName('t.product_id') . ' = ' . $db->quoteName('p.id') . ')')
          ->where($db->quoteName('t.userid') . ' = ' . (int)$userid)
          ->order($db->quoteName($sort) . ' ASC')
         ;

    $db->setQuery($query);
    $result = $db->loadObjectList();
    return $result;
  }

  public static function getDownloadLink($ticketid)
  {
    if(empty($ticketid))
      return '';
		$dest = JPATH_ADMINISTRATOR . '/components/com_lartssubscriptions/tickets/ticket_' . $ticketid . '.pdf';
    return $dest;
  }
  
    public static function getTicketByDownloadKey($key)
	{
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->select('*')
		            ->from($db->quoteName("#__larts_subs_tickets"))
		            ->where($db->quoteName("dlkey"). " = '" .$key. "'");
		$db->setQuery($query);
		return $db->loadObject();
	}

}
