<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace LartsSubscriptions\VAT;
defined('_JEXEC') or die;

use Exception;
use SoapClient;
use SoapFault;
use Date;
use LartsSubscriptions\VAT\VatRates;

// Client for processing request for Tax

class EUTaxClient
{
  public static $countriesInEu = [
        'AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'EL', 'ES', 
        'FI', 'FR', 'HR', 'HU', 'IE', 'IT', 'LT', 'LU', 'LV', 'MT', 
        'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK',
    ];
  private $wsdlRetrieveVAT = 'https://ec.europa.eu/taxation_customs/tedb/ws/VatRetrievalService.wsdl';
  private $wsdlCheckVAT = 'http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl';
	private $webCheckVAT = 'http://ec.europa.eu/taxation_customs/vies/viesquer.do';
  private $soapClient;
  public $error = '';
  public $checkVatRetries = 3;
  public $wsdl;

  public function __construct( $wsdl = null ) {
    if ( $wsdl ) {
      $this->wsdl = $wsdl;
    }
  }

  private function getSoapClient($wsdl)
  {
    if ($this->soapClient == null)
    {
      $this->soapClient = new \SoapClient($wsdl,
        array(
          'user_agent' => 'Mozilla/5.0 (compatible; EU VAT & B2B Taxes)',
          'exceptions' => true,
        )
      );
    }

    return $this->soapClient;
  }

  private function call(string $call, array $params): ?object
  {
    try {
      $response = $this->soapClient->__soapCall($call, $params);
      if (! is_object($response)) {
          return null;
      }
      return $response;
    } catch (SoapFault $fault) {
       throw new Exception($fault->faultstring);
    }
  }

  public function checkVat( $countryCode, $vatNumber )
  {
    $retry = $this->checkVatRetries;
    while($retry) {
      $retry--;
      $result = $this->doCheckVat($countryCode, $vatNumber);
      if(is_object($result)) {
        if($result->valid != true)
          return false;
        return $result;
      }
      switch($this->error) {
      case 'GLOBAL_MAX_CONCURRENT_REQ':
      case 'MS_MAX_CONCURRENT_REQ':
        break;
      default:
        return false;
      }
    };
  }
  
  private function doCheckVat($countryCode, $vatNumber)
  {
    try {
      $soapClient = $this->getSoapClient($this->wsdlCheckVAT);
      $response = $this->call('checkVat', [
        'checkVat' => [
            'countryCode' => $countryCode,
            'vatNumber'   => $vatNumber,
        ],
      ]);

      if ($response === null) {
        return null;
      }
      
      return $response;
      
    } 
    catch ( Exception $e ) {
      $this->error = $e->getMessage();
      return false;
      
    }
  }

  public function retrieveVat() {
    try {
      $soapClient = $this->getSoapClient($this->wsdlRetrieveVAT);

      $date = new \DateTime('now');
      $response = $this->call('retrieveVatRates', [
          'retrieveVatRatesRequest' => [
              'memberStates' => self::$countriesInEu, 
              'situationOn' => $date->format('Y-m-d'),
          ],
      ]);
      if ($response === null) {
        return false;
      }
      
      foreach($response->vatRateResults as $item) {
        
        $record = [
          //'id' => null,
          'country_code' => null,
          'type' => null,
          'category' => null,
          'category_desc' => null,
          'rate' => null,
          'rate_desc' => null,
        ];
        
        $record['country_code'] = $item->memberState;
        switch($item->type) {
        case 'REDUCED':
          $record['type'] = 1;
          break;
        default:
          $record['type'] = 0;
        }
        if(isset($item->category->identifier)) {
          $record['category'] = $item->category->identifier;
          $record['category_desc'] = $item->category->description;
        }
        else if(isset($item->comment)) {
          $record['category_desc'] = $item->comment;
        }
        $record['rate'] = $item->rate->value;
        if(isset($item->cnCodes)) {
          foreach($item->cnCodes as $cnCode) {
            if(is_array($cnCode)) {
              foreach($cnCode as $code) {
                if(strlen($record['rate_desc']) > 0)
                  $record['rate_desc'] .= ', ';
                $record['rate_desc'] .= $code->value;
              }
            }
            else {
              $record['rate_desc'] = $cnCode->value;
            }
          }
        }
        VatRates::addItem($record);
      }
    }
    catch ( Exception $e ) {
      $this->error = $e->getMessage();
      return false;
    }
    return true;
  }
}







