<?php
/**
 * @package         Lartssubscriptions Free
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Component\LartsSubscriptions\Administrator\Model;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Language\Text;

use LartsSubscriptions\Message;
use LartsSubscriptions\Invoice;
use LartsSubscriptions\Base\PluginManager;
use LartsSubscriptions\Periods;
use LartsSubscriptions\Transaction;


class InvoicesModel extends ListModel
{
	var $id         = null;
	var $list       = null;
	var $data       = null;
	var $pagination = null;

	public function __construct($config = [], MVCFactoryInterface $factory = null)
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = [
				'search',
				'country',
				'provider',
				'payment',
				'year',
				'quarter',
			];
		}

		parent::__construct($config, $factory);

		$cid      = Factory::getApplication()->GetInput()->get('cid', [0], 'array');
		$this->id = (int) $cid[0];
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$this->setState('filter.search', $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', null, 'string'));
		$this->setState('filter.provider', $this->getUserStateFromRequest($this->context . '.filter.provider', 'filter_provider', null, 'string'));
		$this->setState('filter.payment', $this->getUserStateFromRequest($this->context . '.filter.payment', 'filter_payment', null, 'string'));
		$this->setState('filter.country', $this->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null, 'int'));
		$this->setState('filter.year', $this->getUserStateFromRequest($this->context . '.filter.year', 'filter_year', null, 'int'));
		$this->setState('filter.quarter', $this->getUserStateFromRequest($this->context . '.filter.quarter', 'filter_quarter', null, 'int'));

		parent::populateState('i.id', 'desc');
	}

	public function getPagination()
	{
		if(!is_null($this->pagination))
		{
			return $this->pagination;
		}

		$this->pagination = new Pagination($this->getTotal(), $this->getState('list.start'), $this->getState('list.limit'));

		return $this->pagination;
	}

	public function getTotal()
	{
		$query = $this->_buildListQuery();

		return $this->_getListCount($query);
	}

	private function _buildListQuery()
	{
		$search   = $this->getUserStateFromRequest($this->context . 'filter.search', 'filter_search', 0, 'string');
		$provider = $this->getUserStateFromRequest($this->context . '.filter.provider', 'filter_provider', null);
		$payment  = $this->getUserStateFromRequest($this->context . '.filter.payment', 'filter_payment', null);
		$country  = $this->getUserStateFromRequest($this->context . '.filter.country', 'filter_country', null);
		$year     = $this->getUserStateFromRequest($this->context . '.filter.year', 'filter_year', null);
		$quarter  = $this->getUserStateFromRequest($this->context . '.filter.quarter', 'filter_quarter', null);

		$ordering  = $this->getUserStateFromRequest($this->context . '.list.ordering', 'list_ordering', 'i.id', 'string');
		$direction = $this->getUserStateFromRequest($this->context . '.list.direction', 'list_direction', 'desc', 'string');

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['i.*', 
                          'u.address1', 'u.zipcode', 'u.city', 'u.lastname', 'u.firstname', 'u.country_code', 'u.vat_number AS vat',
                          't.type'])
		            ->from($db->quoteName('#__larts_subs_invoices', 'i'))
		            ->join('LEFT OUTER', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('i.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_countries', 'c') . ' ON (' . $db->quoteName('u.country_code') . ' = ' . $db->quoteName('c.country_2_code') . ')')
		            ->join('LEFT OUTER', $db->quoteName('#__larts_subs_transactions', 't') . ' ON (' . $db->quoteName('t.transaction_id') . ' = ' . $db->quoteName('i.transaction_id') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'users') . ' ON (' . $db->quoteName('u.userid') . ' = ' . $db->quoteName('users.id') . ')')
            ;

		if ($search)
		{
			$like_filter = ' LIKE ' . $db->quote('%' . str_replace(' ', '%', $search) . '%');

			$fullname = $query->concatenate([$db->quoteName('u.firstname'), $db->quote(' '), $db->quoteName('u.lastname')]);

			$where = [
				$fullname . $like_filter,
				$db->quoteName('u.companyname') . $like_filter,
				$db->quoteName('users.username') . $like_filter,
				$db->quoteName('users.email') . $like_filter,
				$db->quoteName('i.transaction_id') . ' = ' . $db->quote($search),
			];

			$search = preg_replace('#^i-#i', '', $search);

			if (is_numeric($search))
			{
				$where[] = $db->quoteName('i.id') . ' = ' . (int) $search;
				$where[] = $db->quoteName('i.ordercode') . ' = ' . (int) $search;
			}

			$query->where('(' . implode(' OR ', $where) . ')');
		}

		if(!empty($provider))
		{
			$query->where($db->quoteName('t.type') . ' = ' . $db->quote($provider));
		}

		if (is_numeric($payment))
		{
			if ($payment == 3)
			{
				$query->where($db->quoteName('i.refund_invoice') . ' > 0');
			}
			else
			{
				$query->where($db->quoteName('i.published') . ' = ' . (int) $payment);
			}
		}

		if (is_numeric($country))
		{
			$query->where($db->quoteName('c.id') . ' = ' . $country);
		}

		if (is_numeric($year))
		{

			$query->where('YEAR( ' . $db->quoteName('i.invoicedate') . ' ) = ' . $year);
		}

		if (is_numeric($quarter))
		{
			if ($quarter == 1)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  1 AND 3');

			if ($quarter == 2)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  4 AND 6');

			if ($quarter == 3)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  7 AND 9');

			if ($quarter == 4)
				$query->where('MONTH(' . $db->quoteName('i.invoicedate') . ') BETWEEN  10 AND 12');
		}

		$query->group($db->quoteName('i.id'))
		      ->order($db->quoteName($ordering) . ' ' . $direction);

		return $query;
	}

	public function getList()
	{
		if(!is_null($this->list)) {
			return $this->list;
		}

		$db    = Factory::getDbo();
		$query = $this->_buildListQuery();

		$db->setQuery($query, $this->getState('list.start'), $this->getState('list.limit'));
		$this->list = $db->loadObjectList();

		return $this->list;
	}

	public function getData()
	{
		if(!is_null($this->data)) {
			return $this->data;
		}

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            'i.*', 
                  'u.companyname', 'u.firstname', 'u.lastname', 'u.vat_number', 'u.zipcode', 'u.city', 'u.address1', 'u.country_code', 
                  'juser.email',
		            ])
		            ->from($db->quoteName('#__larts_subs_invoices', 'i'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_transactions', 't') . ' ON (' . $db->quoteName('i.ordercode') . ' = ' . $db->quoteName('t.ordercode') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_customers', 'u') . ' ON (' . $db->quoteName('i.userid') . ' = ' . $db->quoteName('u.userid') . ')')
		            ->join('LEFT', $db->quoteName('#__users', 'juser') . ' ON (' . $db->quoteName('i.userid') . ' = ' . $db->quoteName('u.id') . ')')
		            ->where($db->quoteName('i.id') . ' = ' . (int) $this->id);

		$db->setQuery($query);

		$this->data = $db->loadObject();

		if ($this->data->invoice_type == 1)
		{
			$query = $db->getQuery(true)
			            ->select(['id as invoice_id', 'invoice_number'])
			            ->from($db->quoteName('#__larts_subs_invoices'))
			            ->where($db->quoteName('id') . ' = ' . (int) $this->data->refund_invoice);

			$db->setQuery($query);
			$invoice = $db->loadObject();

			$this->data->refund_invoice = ! empty($invoice->invoice_number) ? $invoice->invoice_number : $invoice->invoice_id;
		}

		return $this->data;
	}

	public function creditInvoice($id, $reason = null, $send_email = true)
	{
		$helper = new Invoice;
		$date = new Date;

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['i.*', 't.plugin'])
		            ->from($db->quoteName('#__larts_subs_invoices', 'i'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_transactions', 't') . ' ON (' . $db->quoteName('t.ordercode') . ' = ' . $db->quoteName('i.ordercode') . ')')
		            ->where($db->quoteName('i.id') . ' = ' . (int) $id)
		            ->andWhere($db->quoteName('t.status') . ' = ' . Transaction::PROCESSED)
                    ;

		$db->setQuery($query);
		$data = $db->loadObject();
    if(empty($data)) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_INVOICE_NOT_FOUND'), 'error');
      return false;
    }

		// Prepare some credit invoice data to save it:
		$invoice = [
      'invoice_number' => $helper->generateInvoiceNumber(),
			'gross_price'    => -1 * abs($data->gross_price),
			'net_price'      => -1 * abs($data->net_price),
			'disco_price'    => -1 * abs($data->disco_price),
			'vat_total'      => -1 * abs($data->vat_total),
			'vat_percentage' => '',
			'invoice_type'   => Invoice::TYPE_CREDIT,
			'ordercode'      => $data->ordercode,
			'userid'         => $data->userid,
			'paid'           => Invoice::UNPAID,
			'refund_reason'  => $reason,
			'refund_invoice' => $id,
			'provider'       => $data->provider,
			'published'      => ($send_email ? 1 : 0),
      'refund_date'    => $date->toSql(),
		];

		$invoice_id = $this->store($invoice);

		$query = $db->getQuery(true)
		            ->select('id')
		            ->from($db->quoteName('#__larts_subs_periods'))
		            ->where($db->quoteName('ordercode') . ' = ' . (int) $data->ordercode);

		$db->setQuery($query);
		$ids = $db->loadColumn();

		$arguments = [
			'transaction_id' => $data->transaction_id,
			'ordercode'      => $data->ordercode,
			'amount'         => $data->net_price,
			'refund_invoice' => $id,
		];

		// initiate the refund at the PSP
    $bRet = PluginManager::execPlugin('lartssubscriptions_payment', strtolower($data->plugin), 'onRefund', $arguments);
    $bRet = $bRet[0];
    if(!$bRet) {
      Factory::getApplication()->enqueueMessage(Text::_('COM_LARTSSUBS_REFUND_NOT_PROCESSED'), 'warning');
    }
    else {
      Invoice::setPaid($data->ordercode, Invoice::PAID);
    }

		$cid = ArrayHelper::toInteger($ids);

		PluginManager::execPlugins('lartssubscriptions', 'onBeforeRemoveSubscription', $cid);

    // change orig invoice state to REFUNDED
    Transaction::setRefunded($data->transaction_id);

    // mark period as refunded
    $query = $db->getQuery(true)
		            ->update($db->quoteName('#__larts_subs_periods'))
		            ->set($db->quoteName('status') . ' = '. Periods::STATE_REFUNDED)
		            ->where($db->quoteName('ordercode') . ' = ' . (int) $data->ordercode);

		$db->setQuery($query);
		$db->execute();

		PluginManager::execPlugins('lartssubscriptions', 'onAfterRemoveSubscription', $ids);

		$showInvoice = new Invoice();
		$showInvoice->createInvoicePdf($invoice_id, true);

		if ($send_email) {
			//$this->sendCreditInvoice($invoice_id);
			$this->sendInvoice($invoice_id, 'send-refund-message');
		}

		return true;
	}

//	public function sendCreditInvoice($id)
//	{
//		$db = Factory::getDbo();
//
//		$query = $db->getQuery(true)
//		            ->select(['id as invoice_id', 'net_price as price', 'invoicedate as date', 'userid', 'invoice_number'])
//		            ->from($db->quoteName('#__larts_subs_invoices'))
//		            ->where($db->quoteName('id') . ' = ' . (int) $id);
//
//		$db->setQuery($query);
//		$data = $db->loadObject();
//
//		// Make sure the invoice price is not negative
//		//if ($data->price < 0) {
//		//	$data->price = $data->price * -1;
//		//}
//
//    $invoice = new Invoice;
//		$attachment = $invoice->getPath() . $data->invoice_number . '.pdf';
//
//		Message::getInstance('send-refund-message')
//		             ->variables($data)
//		             ->attachment($attachment)
//		             ->send();
//
//    $invoice->setPublished($id, true);
//
//		return true;
//	}

	public function send(&$pks)
	{
		if (empty($pks)) {
			return true;
		}

		$pks  = ArrayHelper::toInteger($pks);
		$cids = implode(',', $pks);

		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select(['id', 'ordercode', 'userid', 'published'])
		            ->from($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('id') . ' IN  ( ' . $cids . ' )');
		$db->setQuery($query);
		$data = $db->loadObjectList();

		foreach ($data as $row)
		{
			$showInvoice = new Invoice();
			$showInvoice->createInvoicePdf((int) $row->id);

			self::sendInvoice($row->id);

			unset($row);
		}

		return true;
    
  }
  
	public function sendInvoice($id, $alias = 'send-invoice')
	{
    $invoice = new Invoice;
    
    $data = $invoice->getInvoiceById($id);
    
		$attachment = $invoice->getPath() . $data->invoice_number . '.pdf';

		Message::getInstance($alias)
		             ->invoice($id)
		             ->attachment($attachment)
		             ->send();

    $invoice->setPublished($id, true);

    return true;
	}

	public function store($data)
	{
		$row = $this->getTable();

		if(!$row->bind($data))
		{
			return false;
		}

		if(!$row->check())
		{
			return false;
		}

		if(!$row->store())
		{
			return false;
		}

		$this->id = $this->_db->insertid() ?: Factory::getApplication()->GetInput()->getInt('id');

		return $this->id;
	}

	public function getFormExport($data = [], $loadData = false)
	{
		return $this->loadForm('com_lartssubscriptions.export', 'export', ['load_data' => $loadData]);
	}

	public function getForm($data = [], $loadData = true)
	{
		return $this->loadForm('com_lartssubscriptions.invoice', 'invoice', ['load_data' => $loadData]);
	}

	protected function loadFormData()
	{
		$task = Factory::getApplication()->GetInput()->getString('task');

		if ($task === 'edit')
		{
			return $this->getData();
		}

		return parent::loadFormData();
	}

	public function getProducts()
	{
		$db = Factory::getDbo();

		$query = $db->getQuery(true)
		            ->select([
			            'p.id', 'p.name',
			            'p.period', 'p.period_type',
			            'COUNT( p.id ) AS count_products',
                  'o.licence',
		            ])
		            ->from($db->quoteName('#__larts_subs_cart_items', 'o'))
		            ->join('LEFT', $db->quoteName('#__larts_subs_products', 'p') . ' ON (' . $db->quoteName('p.id') . ' = ' . $db->quoteName('o.product_id') . ')')
		            ->join('LEFT', $db->quoteName('#__larts_subs_invoices', 'i') . ' ON (' . $db->quoteName('i.ordercode') . ' = ' . $db->quoteName('o.ordercode') . ')')
		            ->where($db->quoteName('i.id') . ' = ' . (int) $this->id)
		            ->order($db->quoteName('p.name'))
		            ->group($db->quoteName('p.id'));

		$db->setQuery($query);

		return $db->loadObjectList();
	}

	/**
	 * Removing invoices from the database.
	 *
	 * @param $cid
	 *
	 * @return bool
	 */
	public function remove($cid)
	{
		if (empty($cid)){
			return true;
		}

		// remove the pdfs from server
    $invoice = new Invoice;
		$invoice->removeInvoicePDFFiles($cid);

		$cid  = ArrayHelper::toInteger($cid);
		$cids = implode(',', $cid);

		// remove the invoices from db
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
		            ->delete($db->quoteName('#__larts_subs_invoices'))
		            ->where($db->quoteName('id') . ' IN ' . '( ' . $cids . ' )');
		$db->setQuery($query);

		return $db->execute();
	}

}
